@@ -120,15 +120,18 @@ import GI.Vte
   , terminalSetFont
   , terminalSetScrollbackLines
   , terminalSetWordCharExceptions
+  , terminalSetAllowBold
   )
 import System.Environment (getExecutablePath)
 import System.FilePath (takeFileName)
 
 import Paths_termonad (getDataFileName)
-import Termonad.Gtk (appNew, objFromBuildUnsafe)
+import Termonad.Gtk (appNew, objFromBuildUnsafe, terminalSetEnableSixelIfExists)
 import Termonad.Keys (handleKeyPress)
 import Termonad.Lenses
   ( lensBoldIsBright
+  , lensEnableSixel
+  , lensAllowBold
   , lensConfirmExit
   , lensCursorBlinkMode
   , lensFontConfig
@@ -727,6 +730,8 @@ applyNewPreferencesToTab mvarTMState tab = do
   terminalSetWordCharExceptions term (wordCharExceptions options)
   terminalSetScrollbackLines term (fromIntegral (scrollbackLen options))
   terminalSetBoldIsBright term (boldIsBright options)
+  terminalSetEnableSixelIfExists term (enableSixel options)
+  terminalSetAllowBold term (allowBold options)
 
   let vScrollbarPolicy = showScrollbarToPolicy (options ^. lensShowScrollbar)
   scrolledWindowSetPolicy scrolledWin PolicyTypeAutomatic vScrollbarPolicy
@@ -752,6 +757,10 @@ showPreferencesDialog mvarTMState = do
     objFromBuildUnsafe preferencesBuilder "showMenu" CheckButton
   boldIsBrightCheckButton <-
     objFromBuildUnsafe preferencesBuilder "boldIsBright" CheckButton
+  enableSixelCheckButton <-
+    objFromBuildUnsafe preferencesBuilder "enableSixel" CheckButton
+  allowBoldCheckButton <-
+    objFromBuildUnsafe preferencesBuilder "allowBold" CheckButton
   wordCharExceptionsEntryBuffer <-
     objFromBuildUnsafe preferencesBuilder "wordCharExceptions" Entry >>=
       getEntryBuffer
@@ -806,6 +815,8 @@ showPreferencesDialog mvarTMState = do
   toggleButtonSetActive confirmExitCheckButton $ confirmExit options
   toggleButtonSetActive showMenuCheckButton $ showMenu options
   toggleButtonSetActive boldIsBrightCheckButton $ boldIsBright options
+  toggleButtonSetActive enableSixelCheckButton $ enableSixel options
+  toggleButtonSetActive allowBoldCheckButton $ allowBold options
   entryBufferSetText wordCharExceptionsEntryBuffer (wordCharExceptions options) (-1)
 
   -- Run dialog then close
@@ -827,6 +838,8 @@ showPreferencesDialog mvarTMState = do
     confirmExitVal <- toggleButtonGetActive confirmExitCheckButton
     showMenuVal <- toggleButtonGetActive showMenuCheckButton
     boldIsBrightVal <- toggleButtonGetActive boldIsBrightCheckButton
+    enableSixelVal <- toggleButtonGetActive enableSixelCheckButton
+    allowBoldVal <- toggleButtonGetActive allowBoldCheckButton
     wordCharExceptionsVal <- entryBufferGetText wordCharExceptionsEntryBuffer
 
     -- Apply the changes to mvarTMState
@@ -836,6 +849,8 @@ showPreferencesDialog mvarTMState = do
         ( set lensConfirmExit confirmExitVal
         . set lensShowMenu showMenuVal
         . set lensBoldIsBright boldIsBrightVal
+        . set lensEnableSixel enableSixelVal
+        . set lensAllowBold allowBoldVal
         . set lensWordCharExceptions wordCharExceptionsVal
         . over lensFontConfig (`fromMaybe` maybeFontConfig)
         . set lensScrollbackLen scrollbackLenVal
