@@ -50,6 +50,8 @@ module Termonad.Config.Colour
     , lensCursorBgColour
     , lensForegroundColour
     , lensBackgroundColour
+    , lensHighlightFgColour
+    , lensHighlightBgColour
     , lensPalette
     -- * Colour Extension
     , ColourExtension(..)
@@ -119,6 +121,8 @@ import GI.Vte
 #endif
   , terminalSetColorBackground
   , terminalSetColorForeground
+  , terminalSetColorHighlight
+  , terminalSetColorHighlightForeground
   )
 import Text.Printf (printf)
 import Text.Show (showString)
@@ -177,7 +181,7 @@ unsafeMkList8 xs =
 -- >>> setAt 100 "hello" ["a","b","c","d"]
 -- ["a","b","c","d"]
 setAt :: forall a. Int -> a -> [a] -> [a]
-setAt n newVal = overAt n (\_ -> newVal)
+setAt n newVal = overAt n (const newVal)
 
 -- | Update a given value in a list.
 --
@@ -280,14 +284,14 @@ newtype Matrix a = Matrix (List6 (List6 (List6 a)))
   deriving (Show, Eq, Functor, Foldable)
 
 getMatrix :: Matrix a -> [[[a]]]
-getMatrix (Matrix (List6 m)) = fmap getList6 $ (fmap . fmap) getList6 m
+getMatrix (Matrix (List6 m)) = fmap (getList6 . fmap getList6) m
 
 -- | Unsafe smart constructor for 6x6x6 Matrices.
 mkMatrix :: [[[a]]] -> Maybe (Matrix a)
 mkMatrix xs =
   if length xs == 6 && all (\x -> length x == 6) xs
                     && all (all (\x -> length x == 6)) xs
-  then Just $ Matrix $ List6 (fmap List6 ((fmap . fmap) List6 xs))
+  then Just $ Matrix $ List6 (fmap (List6 . fmap List6) xs)
   else Nothing
 
 -- | Unsafe smart constructor for 6x6x6 Matrices.
@@ -296,14 +300,14 @@ unsafeMkMatrix xs =
   case mkMatrix xs of
     Just xs' -> xs'
     Nothing ->
-      error $
+      error
         "unsafeMkMatrix: input list must be exactly 6x6x6"
 
 -- | Set a given value in a 'Matrix'.
 --
 -- Internally uses 'setAt'.  See documentation on 'setAt' for some examples.
 setAtMatrix :: Int -> Int -> Int -> a -> Matrix a -> Matrix a
-setAtMatrix x y z a m = overAtMatrix x y z (\_ -> a) m
+setAtMatrix x y z a = overAtMatrix x y z (const a)
 
 -- | Set a given value in a 'Matrix'.
 --
@@ -496,7 +500,7 @@ sRGB32
 sRGB32 r g b 255 = withOpacity (sRGB24 r g b) 1
 sRGB32 r g b a =
   let aDouble = fromIntegral a / 255
-  in (withOpacity (sRGB24 r g b) aDouble)
+  in withOpacity (sRGB24 r g b) aDouble
 
 -- | Create an 'AlphaColour' that is fully 'opaque'.
 --
@@ -672,11 +676,14 @@ defaultGreyscale = unsafeMkList24 $ do
 -- 'foregroundColour' and 'backgroundColour' allow you to set the color of the
 -- foreground text and background of the terminal.
 --
+-- 'highlightFgColour' and 'highlightBgColour' allow you to set the color of
+-- the foreground and background of the highlighted text.
+--
 -- 'palette' allows you to set the full color palette used by the terminal.
 -- See 'Palette' for more information.
 --
--- If you don't set 'foregroundColour', 'backgroundColour', or 'palette', the
--- defaults from VTE are used.
+-- If you don't set 'foregroundColour', 'backgroundColour', 'highlightFgColour',
+-- 'highlightBgColour', or 'palette', the defaults from VTE are used.
 --
 -- If you want to use a terminal with a white (or light) background and a black
 -- foreground, it may be a good idea to change some of the colors in the
@@ -751,22 +758,28 @@ data ColourConfig c = ColourConfig
     -- ^ Color of the default foreground text in the terminal.
   , backgroundColour :: !(Option c)
     -- ^ Background color for the terminal
+  , highlightFgColour :: !(Option c)
+    -- ^ Foreground color for the highlighted text.
+  , highlightBgColour :: !(Option c)
+    -- ^ Background color for the highlighted text.
   , palette :: !(Palette c)
     -- ^ Color palette for the terminal.  See 'Palette'.
   } deriving (Eq, Show, Functor)
 
 -- | Default setting for a 'ColourConfig'.  The cursor colors, font foreground
--- color, background color, and color palette are all left at the defaults set
--- by VTE.
+-- color, background color, highlighted text color, and color palette are all
+-- left at the defaults set by VTE.
 --
 -- >>> defaultColourConfig
--- ColourConfig {cursorFgColour = Unset, cursorBgColour = Unset, foregroundColour = Unset, backgroundColour = Unset, palette = NoPalette}
+-- ColourConfig {cursorFgColour = Unset, cursorBgColour = Unset, foregroundColour = Unset, backgroundColour = Unset, highlightFgColour = Unset, highlightBgColour = Unset, palette = NoPalette}
 defaultColourConfig :: ColourConfig (AlphaColour Double)
 defaultColourConfig = ColourConfig
   { cursorFgColour = Unset
   , cursorBgColour = Unset
   , foregroundColour = Unset
   , backgroundColour = Unset
+  , highlightFgColour = Unset
+  , highlightBgColour = Unset
   , palette = NoPalette
   }
 
@@ -775,6 +788,8 @@ $(makeLensesFor
     , ("cursorBgColour", "lensCursorBgColour")
     , ("foregroundColour", "lensForegroundColour")
     , ("backgroundColour", "lensBackgroundColour")
+    , ("highlightFgColour", "lensHighlightFgColour")
+    , ("highlightBgColour", "lensHighlightBgColour")
     , ("palette", "lensPalette")
     ]
     ''ColourConfig
@@ -813,6 +828,10 @@ colourHook mvarColourConf _ vteTerm = do
   whenSet (cursorFgColour colourConf) $
     terminalSetColorCursorForeground vteTerm . Just <=< colourToRgba
 #endif
+  whenSet (highlightFgColour colourConf) $
+    terminalSetColorHighlightForeground vteTerm . Just <=< colourToRgba
+  whenSet (highlightBgColour colourConf) $
+    terminalSetColorHighlight vteTerm . Just <=< colourToRgba
 
 colourToRgba :: AlphaColour Double -> IO RGBA
 colourToRgba colour = do
@@ -873,4 +892,3 @@ addColourHook
 addColourHook newHook oldHook tmState term = do
   oldHook tmState term
   newHook tmState term
-
